@@ -368,6 +368,7 @@ class Trainer(Fit):
                     cal_pred[PREDICTION_COLUMN] = y_pred[PREDICTION_COLUMN]
                     output = 0.0
                     loss = 0.0
+                    pvalue = 0.0
                     if determine_model_type(y_series) == ModelType.REGRESSION:
                         output = float(r2_score(y_test, y_pred[[PREDICTION_COLUMN]]))
                         print(f"R2: {output}")
@@ -377,10 +378,12 @@ class Trainer(Fit):
                         prob_col = PROBABILITY_COLUMN_PREFIX + str(1)
                         if prob_col in y_pred.columns.values.tolist():
                             loss = float(brier_score_loss(y_test, y_pred[[prob_col]]))
+                            pvalue = calibrator.p_value
                             print(f"Brier: {loss}")
                             print(
                                 f"Log Loss: {float(log_loss(y_test.astype(float), y_pred[[prob_col]]))}"
                             )
+                            print(f"P-Value: {pvalue}")
                         print(
                             f"Accuracy: {float(accuracy_score(y_test, y_pred[[PREDICTION_COLUMN]]))}"
                         )
@@ -504,7 +507,7 @@ class Trainer(Fit):
                         test_series,
                         True,
                         test_idx,
-                        True,
+                        False,
                     )
                     continue
                 if (
@@ -538,7 +541,7 @@ class Trainer(Fit):
                 else:
                     break
 
-                _fit(
+                value = _fit(
                     study.best_trial,
                     test_df.copy(),
                     test_series,
@@ -546,14 +549,23 @@ class Trainer(Fit):
                     test_idx,
                     False,
                 )
-                _fit(
-                    _best_trial(study, dt=test_dt),
+                best_dt_trial = _best_trial(study, dt=test_dt)
+                best_trial = (
+                    study.best_trial
+                    if best_dt_trial is None
+                    or value
+                    < (0.0 if best_dt_trial.value is None else best_dt_trial.value)
+                    else best_dt_trial
+                )
+                value = _fit(
+                    best_trial,
                     test_df.copy(),
                     test_series,
                     True,
                     test_idx,
-                    True,
+                    False,
                 )
+                print(f"Saved model with value: {value}")
                 last_processed_dt = test_idx
 
             target_names = ["F1", "Brier"]
@@ -609,9 +621,12 @@ class Trainer(Fit):
                 date_path = os.path.join(column_path, date_str)
                 if not os.path.isdir(date_path):
                     continue
+                if not os.listdir(date_path):
+                    continue
                 dates.append(datetime.datetime.fromisoformat(date_str))
             if not dates:
                 raise ValueError(f"no dates found for {column}.")
+            dates = sorted(dates)
             bins: list[datetime.datetime] = sorted(
                 [dt_index.min().to_pydatetime()]
                 + dates
@@ -644,33 +659,38 @@ class Trainer(Fit):
                 filtered_dates = [x for x in dates if x < max_date]
                 if not filtered_dates:
                     filtered_dates = [dates[-1]]
-                date_str = dates[-1].isoformat()
+                date_str = filtered_dates[-1].isoformat()
                 folder = os.path.join(column_path, date_str)
+                print(f"Loading {folder}")
 
-                reducer = CombinedReducer(
-                    self.embedding_cols,
-                    self._correlation_chunk_size,
-                    self._insert_nulls,
-                )
-                reducer.load(folder)
+                try:
+                    reducer = CombinedReducer(
+                        self.embedding_cols,
+                        self._correlation_chunk_size,
+                        self._insert_nulls,
+                    )
+                    reducer.load(folder)
 
-                model = ModelRouter(None, None)
-                model.load(folder)
+                    model = ModelRouter(None, None)
+                    model.load(folder)
 
-                selector = Selector(model)
-                selector.load(folder)
+                    selector = Selector(model)
+                    selector.load(folder)
 
-                calibrator = CalibratorRouter(model)
-                calibrator.load(folder)
+                    calibrator = CalibratorRouter(model)
+                    calibrator.load(folder)
+
+                    x_pred = reducer.transform(group[feature_columns])
+                    x_pred = selector.transform(x_pred)
+                    y_pred = model.transform(x_pred)
+                    y_pred = calibrator.transform(
+                        y_pred if calibrator.predictions_as_x(None) else x_pred
+                    )
+                    for new_column in y_pred.columns.values:
+                        group["_".join([column, new_column])] = y_pred[new_column]
+                except FileNotFoundError as exc:
+                    print(str(exc))
 
-                x_pred = reducer.transform(group[feature_columns])
-                x_pred = selector.transform(x_pred)
-                y_pred = model.transform(x_pred)
-                y_pred = calibrator.transform(
-                    y_pred if calibrator.predictions_as_x(None) else x_pred
-                )
-                for new_column in y_pred.columns.values:
-                    group["_".join([column, new_column])] = y_pred[new_column]
                 return group
 
             old_index = dt_index.copy()
@@ -695,7 +715,7 @@ class Trainer(Fit):
         return df
 
     def feature_importances(
-        self, df: pd.DataFrame
+        self, df: pd.DataFrame, latest_date_only: bool = False
     ) -> dict[str, tuple[dict[str, float], list[dict[str, float]]]]:
         """Find the feature importances for the rolling models."""
         feature_importances = {}
@@ -704,10 +724,16 @@ class Trainer(Fit):
             column_path = os.path.join(self._folder, column)
             if not os.path.isdir(column_path):
                 continue
-            for date_str in os.listdir(column_path):
-                date_path = os.path.join(column_path, date_str)
-                if not os.path.isdir(date_path):
-                    continue
+            date_paths = sorted(
+                [
+                    os.path.join(column_path, x)
+                    for x in list(os.listdir(column_path))
+                    if os.path.isdir(os.path.join(column_path, x))
+                ]
+            )
+            if latest_date_only:
+                date_paths = date_paths[-1:]
+            for date_path in tqdm.tqdm(date_paths):
                 try:
                     reducer = CombinedReducer(
                         self.embedding_cols,
@@ -725,7 +751,9 @@ class Trainer(Fit):
                     x_pred = reducer.transform(df)
                     x_pred = selector.transform(x_pred)
 
-                    feature_importances[date_str] = model.feature_importances(x_pred)
+                    feature_importances[date_path.split("/")[-1]] = (
+                        model.feature_importances(x_pred)
+                    )
                 except FileNotFoundError as exc:
                     logging.warning(str(exc))
 
